import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import '@adv-ui/boros-adevinta-plugin';
import './patchAnalytics';
import initTcfTracking from './tcf';
import { getAdobeVisitorData, getAdobeMCVisitorID } from './adobeRepository';
import { getPageReferrer, updatePageReferrer } from './referrer';
import { syncPixels } from './syncPixels';
import { getConfig } from './getConfig';
/* Initialize TCF Tracking with Segment */

initTcfTracking();
/* Default properties to be sent on all trackings */

var DEFAULT_PROPERTIES = {
  platform: 'web'
};
/* ServerSide Forwarding values */

var SSF = {
  enabled: 0,
  disabled: 1
};
/**
 * Get default properties using the constant and the window.__mpi object if available
 * @returns {{[key:string]: any}} Default properties to attach to track
 */

var getDefaultProperties = function getDefaultProperties() {
  return _extends({}, DEFAULT_PROPERTIES, getConfig('defaultProperties'));
};
/**
 * Get user traits from global analytics object and put in the object
 * @param {object} traits Info of the user we already have
 * @returns {object} Traits with new merged info
 */


var mergeUserTraitsTo = function mergeUserTraitsTo(traits) {
  var user = window.analytics.user();
  return _extends({}, traits, {
    anonymousId: user.anonymousId(),
    userId: user.id()
  });
};
/**
 * Get AdobeMarketingCloudVisitorId and merge into the integrations object
 * @param {object} integrations Object with all the integrations
 * @returns {Promise<object>} Integrations with the merged Adobe Analytics needed info
 */


var mergeAdobeAnalyticsIntegrationsTo = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(integrations) {
    var marketingCloudVisitorId;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (integrations === void 0) {
              integrations = {};
            }

            _context.next = 3;
            return getAdobeMCVisitorID();

          case 3:
            marketingCloudVisitorId = _context.sent;

            if (marketingCloudVisitorId) {
              _context.next = 6;
              break;
            }

            return _context.abrupt("return", integrations);

          case 6:
            return _context.abrupt("return", _extends({}, integrations, {
              'Adobe Analytics': {
                marketingCloudVisitorId: marketingCloudVisitorId
              }
            }));

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function mergeAdobeAnalyticsIntegrationsTo(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Get data like traits and integrations to be added to the context object
 * @param {object} context Context object with all the actual info
 * @returns {Promise<object>} New context with all the previous info and the new one
 */


var decorateContextWithNeededData = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(context) {
    var _yield$Promise$all, integrations, _yield$Promise$all$, isDMPReady;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (context === void 0) {
              context = {};
            }

            _context2.next = 3;
            return Promise.all([mergeAdobeAnalyticsIntegrationsTo(context.integrations) // checkIsDMPReady()
            ]);

          case 3:
            _yield$Promise$all = _context2.sent;
            integrations = _yield$Promise$all[0];
            _yield$Promise$all$ = _yield$Promise$all[1];
            isDMPReady = _yield$Promise$all$ === void 0 ? true : _yield$Promise$all$;
            return _context2.abrupt("return", _extends({}, context, {
              integrations: integrations,
              page: _extends({}, context.page, {
                referrer: getPageReferrer()
              }),
              traits: _extends({
                ssf: isDMPReady ? SSF.enabled : SSF.disabled
              }, mergeUserTraitsTo(context.traits))
            }));

          case 8:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function decorateContextWithNeededData(_x2) {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * The track method lets you record any actions your users perform.
 * @param {string} event The name of the event you’re tracking
 * @param {object} [properties] A dictionary of properties for the event.
 * @param {object} [context] A dictionary of options.
 * @param {function} [callback] A function executed after a short timeout, giving the browser time to make outbound requests first.
 * @returns {Promise}
 */


var track = function track(event, properties, context, callback) {
  if (context === void 0) {
    context = {};
  }

  return new Promise(function (resolve) {
    window.analytics.ready( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
      var newContext, newProperties, newCallback;
      return _regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return decorateContextWithNeededData(context);

            case 2:
              newContext = _context3.sent;
              newProperties = _extends({}, getDefaultProperties(), properties);

              newCallback = function newCallback() {
                for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                  args[_key] = arguments[_key];
                }

                if (callback) callback.apply(void 0, args); // eslint-disable-line standard/no-callback-literal

                getAdobeMCVisitorID().then(function (marketingCloudVisitorId) {
                  syncPixels(marketingCloudVisitorId).then(function () {
                    resolve.apply(void 0, args);
                  });
                });
              };

              window.analytics.track(event, newProperties, newContext, newCallback);

            case 6:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    })));
  });
};
/**
 * Associate your users and their actions to a recognizable userId and traits.
 * @param {string} userId Id to identify the user.
 * @param {object} traits A dictionary of traits you know about the user, like their email or name.
 * @param {object} [options] A dictionary of options.
 * @param {function} [callback] A function executed after a short timeout, giving the browser time to make outbound requests first.
 * @returns {Promise}
 */


var identify = function identify(userId, traits, options, callback) {
  return Promise.resolve(window.analytics.identify(userId, traits, options, callback));
};
/**
 * Record whenever a user sees a page of your website, along with any optional properties about the page.
 * It updates automatically the referrer from the context on SPA navigations.
 * @param {string} event The name of the event you’re tracking
 * @param {object} properties A dictionary of properties for the event.
 * @param {object} [context] A dictionary of options.
 * @param {function} [callback] A function executed after a short timeout, giving the browser time to make outbound requests first.
 * @returns {Promise}
 */


var page = function page(event, properties, context, callback) {
  if (context === void 0) {
    context = {};
  }

  // update the page referrer before tracking the event
  updatePageReferrer();
  return track(event, properties, context, callback);
};
/**
 * Resets the id, including anonymousId, and clear traits for the currently identified user and group.
 * NOTE: Only clears the cookies and localStorage set by analytics.
 * @returns {Promise}
 */


var reset = function reset() {
  return Promise.resolve(window.analytics.reset());
};

export { getAdobeVisitorData, getAdobeMCVisitorID, decorateContextWithNeededData };
export default {
  track: track,
  identify: identify,
  page: page,
  reset: reset
};